package org.LiveGraph.gui;

import java.awt.Color;
import java.awt.ComponentOrientation;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.Box;
import javax.swing.ButtonGroup;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.WindowConstants;
import java.awt.Dimension;
import javax.swing.JLabel;
import javax.swing.JButton;

import javax.swing.BorderFactory;
import javax.swing.border.EtchedBorder;

import org.LiveGraph.LiveGraph;
import org.LiveGraph.dataCache.CacheObserver;
import org.LiveGraph.dataCache.DataCache;
import org.LiveGraph.settings.GraphSettings;
import org.LiveGraph.settings.ObservableSettings;
import org.LiveGraph.settings.SettingsObserver;

import com.softnetConsult.utils.collections.ReadOnlyIterator;
import com.softnetConsult.utils.swing.DisEnablingPanel;
import com.softnetConsult.utils.swing.ResizablePopupComboBox;
import com.softnetConsult.utils.swing.SwingTools;


import static org.LiveGraph.settings.GraphSettings.HGridType.*;
import static org.LiveGraph.settings.GraphSettings.VGridType.*;
import static org.LiveGraph.settings.GraphSettings.XAxisType.*;


/**
 * The "Graph Settings" window of the application.
 * 
 * <p style="font-size:smaller;">This product includes software developed by the
 *    <strong>LiveGraph</strong> project and its contributors.<br />
 *    (<a href="http://www.live-graph.org" target="_blank">http://www.live-graph.org</a>)<br />
 *    Copyright (c) 2007 G. Paperin.<br />
 *    All rights reserved.
 * </p>
 * <p style="font-size:smaller;">File: GraphSettingsWindow.java</p> 
 * <p style="font-size:smaller;">Redistribution and use in source and binary forms, with or
 *    without modification, are permitted provided that the following terms and conditions are met:
 * </p>
 * <p style="font-size:smaller;">1. Redistributions of source code must retain the above
 *    acknowledgement of the LiveGraph project and its web-site, the above copyright notice,
 *    this list of conditions and the following disclaimer.<br />
 *    2. Redistributions in binary form must reproduce the above acknowledgement of the
 *    LiveGraph project and its web-site, the above copyright notice, this list of conditions
 *    and the following disclaimer in the documentation and/or other materials provided with
 *    the distribution.<br />
 *    3. All advertising materials mentioning features or use of this software or any derived
 *    software must display the following acknowledgement:<br />
 *    <em>This product includes software developed by the LiveGraph project and its
 *    contributors.<br />(http://www.live-graph.org)</em><br />
 *    4. All advertising materials distributed in form of HTML pages or any other technology
 *    permitting active hyper-links that mention features or use of this software or any
 *    derived software must display the acknowledgment specified in condition 3 of this
 *    agreement, and in addition, include a visible and working hyper-link to the LiveGraph
 *    homepage (http://www.live-graph.org).
 * </p>
 * <p style="font-size:smaller;">THIS SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY
 *    OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND  NONINFRINGEMENT. IN NO EVENT SHALL
 *    THE AUTHORS, CONTRIBUTORS OR COPYRIGHT  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING  FROM, OUT OF OR
 *    IN CONNECTION WITH THE SOFTWARE OR THE USE OR  OTHER DEALINGS IN THE SOFTWARE.
 * </p>
 * 
 * @author Greg Paperin (<a href="http://www.paperin.org" target="_blank">http://www.paperin.org</a>)
 * @version {@value org.LiveGraph.LiveGraph#version}
 */
public class GraphSettingsWindow extends JFrame implements SettingsObserver, CacheObserver {

private JTextField minYField = null, maxYField = null, minXField = null, maxXField = null;
private JLabel minYLabelExpl = null, maxYLabelExpl = null, minXLabelExpl = null, maxXLabelExpl = null; 
private JLabel minYLabelAuto = null, maxYLabelAuto = null, minXLabelAuto = null, maxXLabelAuto = null;
private JRadioButton minYValButton = null, minYAutoButton = null, maxYValButton = null, maxYAutoButton = null;
private JRadioButton minXValButton = null, minXAutoButton = null, maxXValButton = null, maxXAutoButton = null;

private JTextField vGridSizeField = null, hGridSizeField = null;
private JRadioButton vGridNoneButton = null, vGridDSNumAlignedButton = null, vGridXAUnitAlignedButton = null;
private JRadioButton hGridNoneButton = null, hGridSimpleButton = null;
private JButton vGridColourButton = null;
private JButton hGridColourButton = null;
private DisEnablingPanel hGridDetsPanel = null, vGridDetsPanel = null;

private JTextField xScaleValueField = null;
private ResizablePopupComboBox seriesCombo = null;
private int preferredXAxisSeriesIndex = 0;
private DisEnablingPanel xAxisSeriesPanel = null;

private JRadioButton xAxisDSNumButton = null, xAxisDataValSimpleButton = null,
					 xAxisDataValTrans01Button = null, xAxisDataValScaledSetButton = null;


/**
 * This is the default constructor.
 */
public GraphSettingsWindow() {
	super();
	initialize();
}


/**
 * This method initializes the window.
 */
private void initialize() {
	
	// Window settings:
	
	final GraphSettingsWindow GRAPHSETTINGS_WIN = this;
	this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	Dimension frameDim = new Dimension(470, 400);
	this.setPreferredSize(frameDim);
	this.setBounds(5, 310, frameDim.width, frameDim.height);	
	this.setTitle("Graph settings (LiveGraph)");
	getContentPane().setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
	//getContentPane().setLayout(new BoxLayout(getContentPane(), BoxLayout.Y_AXIS));
	getContentPane().setComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
	
	// Hide-show listener:
	
	this.addWindowListener(new WindowAdapter() {
		@Override public void windowClosing(WindowEvent e) {
			LiveGraph.application().setDisplayGraphSettingsWindow(false);
		}
	});
	
	// Layout:
	
	GraphSettings settings = LiveGraph.application().getGraphSettings();
	if (null == settings)
		settings = new GraphSettings();
	
	//AbstractButton button = null;
	JLabel label = null;
	ButtonGroup bGroup = null;
	Dimension dim = null;
	RealNumFieldValueChangeAdaptor fvalAdpt = null;
	double defVal = Double.NaN;
	
	// Viewport settings:
	JPanel viewportPanel = new JPanel(new GridBagLayout());
	getContentPane().add(viewportPanel);
	viewportPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	viewportPanel.add(new JLabel("Viewport:"), Tools.createGridBagConstraints(0, 0, 5, 1));
	/*
	viewportPanel.add(new Box.Filler((dim = new Dimension(1, 1)), dim, dim),
					  new GridBagConstraints(0, 5, 6, 1, 1, 1,
											 GridBagConstraints.WEST,
											 GridBagConstraints.BOTH,
											 new Insets(0, 0, 0, 0),
											 0, 0));
											 */	
	
	viewportPanel.add(new JSeparator(SwingConstants.VERTICAL),
					  new GridBagConstraints(5, 1, 1, 2, 0, 1,
							  				 GridBagConstraints.NORTHWEST,
							  				 GridBagConstraints.BOTH,
							  				 new Insets(5, 5, 5, 5),
							  				 0, 0));

	
	// Min Y: 
	viewportPanel.add((minYLabelExpl = new JLabel("Min Y:")), Tools.createGridBagConstraints(0, 1, 1, 1));
	minYLabelExpl.setFont(SwingTools.getPlainFont(minYLabelExpl));
	bGroup = new ButtonGroup();
	
	minYValButton = new JRadioButton("", !Double.isNaN(settings.getMinY()));
	viewportPanel.add(minYValButton, Tools.createGridBagConstraints(1, 1, 1, 1));
	bGroup.add(minYValButton);
	defVal = Double.isNaN(settings.getMinY()) ? 0. : settings.getMinY();
	minYField = new JTextField(Double.toString(defVal), 6);
	viewportPanel.add(minYField, Tools.createGridBagConstraints(2, 1, 1, 1));	
	minYField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(defVal) {
		@Override public double valueChanged(JTextField field, double newValue) {        	
        	if (field.isEnabled())
        		LiveGraph.application().getGraphSettings().setMinY(newValue);
        	else 
        		LiveGraph.application().getGraphSettings().setMinY(Double.NaN);
        	return newValue;
        }
	});
	minYField.addKeyListener(fvalAdpt);

	minYLabelAuto = new JLabel("auto");
	minYLabelAuto.setFont(SwingTools.getPlainFont(minYLabelAuto));
	viewportPanel.add(minYLabelAuto, Tools.createGridBagConstraints(4, 1, 1, 1));
	minYValButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMinY(Double.parseDouble(minYField.getText()));
        }
	});
	
	minYAutoButton = new JRadioButton("", Double.isNaN(settings.getMinY()));
	viewportPanel.add(minYAutoButton, Tools.createGridBagConstraints(3, 1, 1, 1));
	bGroup.add(minYAutoButton);
	minYAutoButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMinY(Double.NaN);
        }
	});
	minYField.setEnabled(false);
	
	// Max Y: 
	viewportPanel.add((maxYLabelExpl = new JLabel("Max Y:")), Tools.createGridBagConstraints(6, 1, 1, 1));
	maxYLabelExpl.setFont(SwingTools.getPlainFont(maxYLabelExpl));
	bGroup = new ButtonGroup();
	
	maxYValButton = new JRadioButton("", !Double.isNaN(settings.getMaxY()));
	viewportPanel.add(maxYValButton, Tools.createGridBagConstraints(7, 1, 1, 1));
	bGroup.add(maxYValButton);
	defVal = Double.isNaN(settings.getMaxY()) ? 100. : settings.getMaxY();
	maxYField = new JTextField(Double.toString(defVal), 6);
	viewportPanel.add(maxYField, Tools.createGridBagConstraints(8, 1, 1, 1));
	maxYField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(defVal) {
		@Override public double valueChanged(JTextField field, double newValue) {
        	if (field.isEnabled())
        		LiveGraph.application().getGraphSettings().setMaxY(newValue);
        	else 
        		LiveGraph.application().getGraphSettings().setMaxY(Double.NaN);
        	return newValue;
        }
	});
	maxYField.addKeyListener(fvalAdpt);

	maxYLabelAuto = new JLabel("auto");
	maxYLabelAuto.setFont(SwingTools.getPlainFont(label));
	viewportPanel.add(maxYLabelAuto, Tools.createGridBagConstraints(10, 1, 1, 1));
	maxYValButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMaxY(Double.parseDouble(maxYField.getText()));
        }
	});
	
	maxYAutoButton = new JRadioButton("", Double.isNaN(settings.getMaxY()));
	viewportPanel.add(maxYAutoButton, Tools.createGridBagConstraints(9, 1, 1, 1));
	bGroup.add(maxYAutoButton);
	maxYAutoButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMaxY(Double.NaN);
        }
	});
	maxYField.setEnabled(false);
	
	// Min X: 
	viewportPanel.add((minXLabelExpl = new JLabel("Min X:")), Tools.createGridBagConstraints(0, 2, 1, 1));
	minXLabelExpl.setFont(SwingTools.getPlainFont(minXLabelExpl));
	bGroup = new ButtonGroup();
	
	minXValButton = new JRadioButton("", !Double.isNaN(settings.getMinX()));
	viewportPanel.add(minXValButton, Tools.createGridBagConstraints(1, 2, 1, 1));
	bGroup.add(minXValButton);
	defVal = Double.isNaN(settings.getMinX()) ? -1. : settings.getMinX();
	minXField = new JTextField(Double.toString(defVal), 6);
	viewportPanel.add(minXField, Tools.createGridBagConstraints(2, 2, 1, 1));
	minXField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(defVal) {
		@Override public double valueChanged(JTextField field, double newValue) {
        	if (field.isEnabled())
        		LiveGraph.application().getGraphSettings().setMinX(newValue);
        	else 
        		LiveGraph.application().getGraphSettings().setMinX(Double.NaN);
        	return newValue;
        }
	});
	minXField.addKeyListener(fvalAdpt);

	minXLabelAuto = new JLabel("auto");
	minXLabelAuto.setFont(SwingTools.getPlainFont(minXLabelAuto));
	viewportPanel.add(minXLabelAuto, Tools.createGridBagConstraints(4, 2, 1, 1));
	minXValButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMinX(Double.parseDouble(minXField.getText()));
        }
	});
	
	minXAutoButton = new JRadioButton("", Double.isNaN(settings.getMinX()));
	viewportPanel.add(minXAutoButton, Tools.createGridBagConstraints(3, 2, 1, 1));
	bGroup.add(minXAutoButton);
	minXAutoButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMinX(Double.NaN);
        }
	});
	minXField.setEnabled(false);
	
	// Max X: 
	viewportPanel.add((maxXLabelExpl = new JLabel("Max X:")), Tools.createGridBagConstraints(6, 2, 1, 1));
	maxXLabelExpl.setFont(SwingTools.getPlainFont(maxXLabelExpl));
	bGroup = new ButtonGroup();
	
	maxXValButton = new JRadioButton("", !Double.isNaN(settings.getMaxX()));
	viewportPanel.add(maxXValButton, Tools.createGridBagConstraints(7, 2, 1, 1));
	bGroup.add(maxXValButton);
	defVal = Double.isNaN(settings.getMaxX()) ? 10000. : settings.getMaxX();
	maxXField = new JTextField(Double.toString(defVal), 6);
	viewportPanel.add(maxXField, Tools.createGridBagConstraints(8, 2, 1, 1));
	maxXField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(defVal) {
		@Override public double valueChanged(JTextField field, double newValue) {
        	if (field.isEnabled())
        		LiveGraph.application().getGraphSettings().setMaxX(newValue);
        	else 
        		LiveGraph.application().getGraphSettings().setMaxX(Double.NaN);
        	return newValue;
        }
	});
	maxXField.addKeyListener(fvalAdpt);

	maxXLabelAuto = new JLabel("auto");
	maxXLabelAuto.setFont(SwingTools.getPlainFont(maxXLabelAuto));
	viewportPanel.add(maxXLabelAuto, Tools.createGridBagConstraints(10, 2, 1, 1));
	maxXValButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMaxX(Double.parseDouble(maxXField.getText()));
        }
	});
	
	maxXAutoButton = new JRadioButton("", Double.isNaN(settings.getMaxX()));
	viewportPanel.add(maxXAutoButton, Tools.createGridBagConstraints(9, 2, 1, 1));
	bGroup.add(maxXAutoButton);
	maxXAutoButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setMaxX(Double.NaN);
        }
	});
	maxXField.setEnabled(false);
	
	// Grid settings:
	JPanel gridPanel = new JPanel(new GridBagLayout());
	getContentPane().add(gridPanel);
	gridPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	
	gridPanel.add(new JSeparator(SwingConstants.VERTICAL),
				  new GridBagConstraints(3, 1, 1, 4, 0, 1,
					  					 GridBagConstraints.NORTHWEST,
					  					 GridBagConstraints.BOTH,
					  					 new Insets(5, 10, 5, 10),
					  					 0, 0));
	
	// Vertical grid:
	gridPanel.add(new JLabel("Vertical grid:"), Tools.createGridBagConstraints(0, 0, 3, 1));	
	bGroup = new ButtonGroup();
	
	vGridDetsPanel = new DisEnablingPanel(new GridBagLayout());
	gridPanel.add(vGridDetsPanel, Tools.createGridBagConstraints(1, 4, 1, 1));
	
	vGridNoneButton = new JRadioButton("Do not display a grid",
									   settings.getVGridType() == VGrid_None);
	vGridNoneButton.setFont(SwingTools.getPlainFont(vGridNoneButton));
	gridPanel.add(vGridNoneButton, Tools.createGridBagConstraints(0, 1, 3, 1));
	bGroup.add(vGridNoneButton);
	vGridNoneButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setVGridType(VGrid_None);
        }
	});
	
	vGridDSNumAlignedButton = new JRadioButton("Grid aligned on dataset indices",
											   settings.getVGridType() == VGrid_DSNumAligned);
	vGridDSNumAlignedButton.setFont(SwingTools.getPlainFont(vGridDSNumAlignedButton));
	gridPanel.add(vGridDSNumAlignedButton, Tools.createGridBagConstraints(0, 2, 3, 1));
	bGroup.add(vGridDSNumAlignedButton);
	vGridDSNumAlignedButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setVGridType(VGrid_DSNumAligned);
        }
	});
	
	vGridXAUnitAlignedButton = new JRadioButton("Grid aligned on X-axis units",
												settings.getVGridType() == VGrid_XAUnitAligned);
	vGridXAUnitAlignedButton.setFont(SwingTools.getPlainFont(vGridXAUnitAlignedButton));
	gridPanel.add(vGridXAUnitAlignedButton, Tools.createGridBagConstraints(0, 3, 3, 1));
	bGroup.add(vGridXAUnitAlignedButton);
	vGridXAUnitAlignedButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setVGridType(VGrid_XAUnitAligned);
        }
	});
	
	dim = new Dimension(20, 10);
	gridPanel.add(new Box.Filler(dim, dim, dim), Tools.createGridBagConstraints(0, 4, 1, 1));
	
	label = new JLabel("Grid size:");
	label.setFont(SwingTools.getPlainFont(label));
	vGridDetsPanel.add(label, Tools.createGridBagConstraints(0, 0, 1, 1));
	
	vGridSizeField = new JTextField(Double.toString(settings.getVGridSize()), 4);
	vGridDetsPanel.add(vGridSizeField, Tools.createGridBagConstraints(1, 0, 1, 1));
	vGridSizeField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(settings.getVGridSize()) {
		@Override public double valueChanged(JTextField field, double newValue) {        	
        	LiveGraph.application().getGraphSettings().setVGridSize(newValue);
        	return LiveGraph.application().getGraphSettings().getVGridSize();
        }
	});
	vGridSizeField.addKeyListener(fvalAdpt);
	
	label = new JLabel("Grid colour:");
	label.setFont(SwingTools.getPlainFont(label));
	vGridDetsPanel.add(label, Tools.createGridBagConstraints(0, 1, 1, 1));
	
	vGridColourButton = new JButton();
	vGridColourButton.setBackground(settings.getVGridColour());
	dim = new Dimension(35, 15);
	vGridColourButton.setSize(dim);
	vGridColourButton.setPreferredSize(dim);
	vGridDetsPanel.add(vGridColourButton, Tools.createGridBagConstraints(1, 1, 1, 1));
	vGridColourButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			Color nc = JColorChooser.showDialog(GRAPHSETTINGS_WIN, "Choose a colour for the vertical grid:",
												LiveGraph.application().getGraphSettings().getHGridColour());
			if (null == nc)
				return;
			LiveGraph.application().getGraphSettings().setVGridColour(nc);
		}
	});
	vGridDetsPanel.setEnabled(false);
	
	// Horizontal grid:
	gridPanel.add(new JLabel("Horizontal grid:"), Tools.createGridBagConstraints(4, 0, 3, 1));	
	bGroup = new ButtonGroup();
	
	hGridDetsPanel = new DisEnablingPanel(new GridBagLayout());
	gridPanel.add(hGridDetsPanel, Tools.createGridBagConstraints(5, 4, 1, 1));
	
	hGridNoneButton = new JRadioButton("Do not display a grid",
									   settings.getHGridType() == HGrid_None);
	hGridNoneButton.setFont(SwingTools.getPlainFont(hGridNoneButton));
	gridPanel.add(hGridNoneButton, Tools.createGridBagConstraints(4, 1, 3, 1));
	bGroup.add(hGridNoneButton);
	hGridNoneButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setHGridType(HGrid_None);
        }
	});
	
	hGridSimpleButton = new JRadioButton("Display a horizontal grid",
										 settings.getHGridType() == HGrid_Simple);
	hGridSimpleButton.setFont(SwingTools.getPlainFont(hGridSimpleButton));
	gridPanel.add(hGridSimpleButton, Tools.createGridBagConstraints(4, 2, 3, 1));
	bGroup.add(hGridSimpleButton);
	hGridSimpleButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setHGridType(HGrid_Simple);
        }
	});
	
	dim = new Dimension(20, 10);
	gridPanel.add(new Box.Filler(dim, dim, dim), Tools.createGridBagConstraints(0, 4, 1, 1));
	
	label = new JLabel("Grid size:");
	label.setFont(SwingTools.getPlainFont(label));
	hGridDetsPanel.add(label, Tools.createGridBagConstraints(0, 0, 1, 1));
	
	hGridSizeField = new JTextField(Double.toString(settings.getHGridSize()), 4);
	hGridDetsPanel.add(hGridSizeField, Tools.createGridBagConstraints(1, 0, 1, 1));
	hGridSizeField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(settings.getHGridSize()) {
		@Override public double valueChanged(JTextField field, double newValue) {        	
        	LiveGraph.application().getGraphSettings().setHGridSize(newValue);
        	return LiveGraph.application().getGraphSettings().getHGridSize();
        }
	});
	hGridSizeField.addKeyListener(fvalAdpt);
	
	label = new JLabel("Grid colour:");
	label.setFont(SwingTools.getPlainFont(label));
	hGridDetsPanel.add(label, Tools.createGridBagConstraints(0, 1, 1, 1));
	
	hGridColourButton = new JButton();
	hGridColourButton.setBackground(Color.LIGHT_GRAY);
	dim = new Dimension(35, 15);
	hGridColourButton.setSize(dim);
	hGridColourButton.setPreferredSize(dim);
	hGridDetsPanel.add(hGridColourButton, Tools.createGridBagConstraints(1, 1, 1, 1));
	hGridColourButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			Color nc = JColorChooser.showDialog(GRAPHSETTINGS_WIN, "Choose a colour for the horizontal grid:",
												LiveGraph.application().getGraphSettings().getHGridColour());
			if (null == nc)
				return;
			LiveGraph.application().getGraphSettings().setHGridColour(nc);
		}
	});
	hGridDetsPanel.setEnabled(false);
	
	//	X axis settings:
	JPanel xAxisPanel = new JPanel(new GridBagLayout());
	xAxisSeriesPanel = new DisEnablingPanel(new GridBagLayout());
	getContentPane().add(xAxisPanel);
	xAxisPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
	xAxisPanel.add(new JLabel("X axis:"), Tools.createGridBagConstraints(0, 0, 4, 1));	
	bGroup = new ButtonGroup();
	/*
	xAxisPanel.add(new Box.Filler((dim = new Dimension(1, 1)), dim, dim),
			  	   new GridBagConstraints(0, 6, 4, 1, 1, 1,
									 	  GridBagConstraints.WEST,
									 	  GridBagConstraints.BOTH,
									 	  new Insets(0, 0, 0, 0),
									 	  0, 0));
									 	  */
	
	xAxisDSNumButton = new JRadioButton("Use dataset number",
										settings.getXAxisType() == XAxis_DSNum);
	xAxisDSNumButton.setFont(SwingTools.getPlainFont(xAxisDSNumButton));
	xAxisPanel.add(xAxisDSNumButton, Tools.createGridBagConstraints(0, 1, 2, 1));
	bGroup.add(xAxisDSNumButton);
	xAxisDSNumButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setXAxisType(XAxis_DSNum);
        }
	});
	
	xAxisDataValSimpleButton = new JRadioButton("Use data series",
												settings.getXAxisType() == XAxis_DataValSimple);
	xAxisDataValSimpleButton.setFont(SwingTools.getPlainFont(xAxisDataValSimpleButton));
	xAxisPanel.add(xAxisDataValSimpleButton, Tools.createGridBagConstraints(0, 2, 2, 1));
	bGroup.add(xAxisDataValSimpleButton);
	xAxisDataValSimpleButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setXAxisType(XAxis_DataValSimple);
        }
	});
	
	xAxisDataValTrans01Button = new JRadioButton("Use data series transformed into [0..1]",
												 settings.getXAxisType() == XAxis_DataValTrans0to1);
	xAxisDataValTrans01Button.setFont(SwingTools.getPlainFont(xAxisDataValTrans01Button));
	xAxisPanel.add(xAxisDataValTrans01Button, Tools.createGridBagConstraints(2, 1, 2, 1));
	bGroup.add(xAxisDataValTrans01Button);
	xAxisDataValTrans01Button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setXAxisType(XAxis_DataValTrans0to1);
        }
	});
	
	xAxisDataValScaledSetButton = new JRadioButton("Use data series scaled by specified value",
												   settings.getXAxisType() == XAxis_DataValScaledSet);
	xAxisDataValScaledSetButton.setFont(SwingTools.getPlainFont(xAxisDataValScaledSetButton));
	xAxisPanel.add(xAxisDataValScaledSetButton, Tools.createGridBagConstraints(2, 2, 2, 1));
	bGroup.add(xAxisDataValScaledSetButton);
	xAxisDataValScaledSetButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphSettings().setXAxisType(XAxis_DataValScaledSet);
        }
	});
	
	dim = new Dimension(20, 10);
	xAxisPanel.add(new Box.Filler(dim, dim, dim), Tools.createGridBagConstraints(0, 3, 1, 1));
	xAxisPanel.add(xAxisSeriesPanel, Tools.createGridBagConstraints(1, 3, 4, 1));
	
	xAxisSeriesPanel.add((label = new JLabel("Series:")), Tools.createGridBagConstraints(0, 0, 1, 1));
	label.setFont(SwingTools.getPlainFont(label));		
	seriesCombo = new ResizablePopupComboBox(new String[] {});
	preferredXAxisSeriesIndex = settings.getXAxisSeriesIndex();	
	xAxisSeriesPanel.add(seriesCombo, Tools.createGridBagConstraints(1, 0, 1, 1));
	seriesCombo.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (0 > seriesCombo.getSelectedIndex())
				return;
			LiveGraph.application().getGraphSettings().setXAxisSeriesIndex(seriesCombo.getSelectedIndex());
        }
	});
	
	xAxisSeriesPanel.add((label = new JLabel("Value:")), Tools.createGridBagConstraints(3, 0, 1, 1));
	label.setFont(SwingTools.getPlainFont(label));	
	xScaleValueField = new JTextField(Double.toString(settings.getXAxisScaleValue()), 6);
	xAxisSeriesPanel.add(xScaleValueField, Tools.createGridBagConstraints(4, 0, 1, 1));
	xScaleValueField.addFocusListener(fvalAdpt = new RealNumFieldValueChangeAdaptor(settings.getXAxisScaleValue()) {
		@Override public double valueChanged(JTextField field, double newValue) {        	
        	LiveGraph.application().getGraphSettings().setXAxisScaleValue(newValue);
        	return LiveGraph.application().getGraphSettings().getXAxisScaleValue();
        }
	});
	xScaleValueField.addKeyListener(fvalAdpt);
	xAxisSeriesPanel.setEnabled(false);
	
	// Set all panes to the same width:
	viewportPanel.setPreferredSize(new Dimension(frameDim.width - 20, viewportPanel.getPreferredSize().height));
	gridPanel.setPreferredSize(new Dimension(frameDim.width - 20, gridPanel.getPreferredSize().height));
	xAxisPanel.setPreferredSize(new Dimension(frameDim.width - 20, xAxisPanel.getPreferredSize().height));
	
	seriesCombo.setPreferredSize(new Dimension(frameDim.width - 250, seriesCombo.getPreferredSize().height));
	
} // private void initialize()


/**
 * Updates the display of the graph settings when they have been changed.
 * @param settings The settings.
 * @param info Event info.
 */
public void settingHasChanged(ObservableSettings settings, Object info) {
	if (null == settings)
		return;
	
	if (settings instanceof GraphSettings) {
		settingHasChanged((GraphSettings) settings, info);
		return;
	}
}

/**
 * Updates the display of the graph settings when they have been changed.
 * @param settings The settings.
 * @param info Event info.
 */
public void settingHasChanged(GraphSettings settings, Object info) {	
	
	if (null == info || !(info instanceof String))
		return;	
	
	String event = (String) info;
	
	if (event.equals("MinY") || event.equals("load")) {
		if (Double.isNaN(settings.getMinY())) {
			minYAutoButton.setSelected(true);
			minYValButton.setSelected(false);
			minYField.setEnabled(false);
		} else {
			minYAutoButton.setSelected(false);
			minYValButton.setSelected(true);
			minYField.setEnabled(true);
			minYField.setText(Double.toString(settings.getMinY()));			
		}
	}
	
	if (event.equals("MaxY") || event.equals("load")) {
		if (Double.isNaN(settings.getMaxY())) {
			maxYAutoButton.setSelected(true);
			maxYValButton.setSelected(false);
			maxYField.setEnabled(false);
		} else {
			maxYAutoButton.setSelected(false);
			maxYValButton.setSelected(true);
			maxYField.setEnabled(true);
			maxYField.setText(Double.toString(settings.getMaxY()));
		}
	}
	
	if (event.equals("MinX") || event.equals("load")) {
		if (Double.isNaN(settings.getMinX())) {
			minXAutoButton.setSelected(true);
			minXValButton.setSelected(false);
			minXField.setEnabled(false);
		} else {	
			minXAutoButton.setSelected(false);
			minXValButton.setSelected(true);
			minXField.setEnabled(true);
			minXField.setText(Double.toString(settings.getMinX()));
		}
	}
	
	if (event.equals("MaxX") || event.equals("load")) {
		if (Double.isNaN(settings.getMaxX())) {
			maxXAutoButton.setSelected(true);
			maxXValButton.setSelected(false);
			maxXField.setEnabled(false);
		} else {
			maxXAutoButton.setSelected(false);
			maxXValButton.setSelected(true);
			maxXField.setEnabled(true);
			maxXField.setText(Double.toString(settings.getMaxX()));		
		}
	}
	
	if (event.equals("VGridType") || event.equals("load")) {
		switch (settings.getVGridType()) {
			case VGrid_None:			vGridDetsPanel.setEnabled(false);
										vGridNoneButton.setSelected(true);
										break;
			case VGrid_DSNumAligned:	vGridDetsPanel.setEnabled(true);
										vGridDSNumAlignedButton.setSelected(true);
										break;
			case VGrid_XAUnitAligned:	vGridDetsPanel.setEnabled(true);
										vGridXAUnitAlignedButton.setSelected(true);
										break;
			default: throw new Error("Greg forgot a VGrid option!");
		}
	}
	
	if (event.equals("VGridSize") || event.equals("load")) {
		vGridSizeField.setText(Double.toString(settings.getVGridSize()));
	}
	
	if (event.equals("VGridColour") || event.equals("load")) {
		vGridColourButton.setBackground(settings.getVGridColour());
	}
	
	if (event.equals("HGridType") || event.equals("load")) {
		switch (settings.getHGridType()) {
			case HGrid_None:	hGridDetsPanel.setEnabled(false);
								hGridNoneButton.setSelected(true);
								break;
			case HGrid_Simple:	hGridDetsPanel.setEnabled(true);
								hGridSimpleButton.setSelected(true);
								break;
			default: throw new Error("Greg forgot a HGrid option!");
		}
	}
	
	if (event.equals("HGridSize") || event.equals("load")) {
		hGridSizeField.setText(Double.toString(settings.getHGridSize()));
	}
	
	if (event.equals("HGridColour") || event.equals("load")) {
		hGridColourButton.setBackground(settings.getHGridColour());
	}
	
	if (event.equals("XAxisType") || event.equals("load")) {
		switch (settings.getXAxisType()) {
			case XAxis_DSNum:				xAxisSeriesPanel.setEnabled(false);
											xAxisDSNumButton.setSelected(true);
											break;
			case XAxis_DataValSimple:		xAxisSeriesPanel.setEnabled(true);
											xAxisDataValSimpleButton.setSelected(true);
											break;
			case XAxis_DataValTrans0to1:	xAxisSeriesPanel.setEnabled(true);
											xAxisDataValTrans01Button.setSelected(true);
											break;
			case XAxis_DataValScaledSet:	xAxisSeriesPanel.setEnabled(true);
											xAxisDataValScaledSetButton.setSelected(true);
											break;
		}
	}
	
	if (event.equals("XAxisSeriesIndex") || event.equals("load")) {
		preferredXAxisSeriesIndex = settings.getXAxisSeriesIndex();
		if (preferredXAxisSeriesIndex < 0)
			preferredXAxisSeriesIndex = 0;
		if (preferredXAxisSeriesIndex < seriesCombo.getItemCount())
			seriesCombo.setSelectedIndex(preferredXAxisSeriesIndex);
		else
			seriesCombo.setSelectedIndex(seriesCombo.getItemCount() - 1);
	}
	
	if (event.equals("XAxisScaleValue") || event.equals("load")) {
		xScaleValueField.setText(Double.toString(settings.getXAxisScaleValue()));
	}
}

/**
 * Updates the series selection drop down box with the specified series labels.
 * @param labels Series labels.
 */
private void setSeriesLabels(ReadOnlyIterator<String> labels) {
	seriesCombo.removeAllItems();
	while (labels.hasNext()) {
		seriesCombo.addItem(labels.next());
	}
	
	if (preferredXAxisSeriesIndex < seriesCombo.getItemCount())
		seriesCombo.setSelectedIndex(preferredXAxisSeriesIndex);
	else
		seriesCombo.setSelectedIndex(seriesCombo.getItemCount() - 1);
}

/**
 * Updates the series labels when the cache is updated.
 */
public void cacheEventFired(DataCache cache, CacheEvent event) {
	
	switch(event) {
		case UpdateLabels:
			setSeriesLabels(cache.iterateDataSeriesLabels());
			break;
		case ChangeMode:
		case UpdateData:
		case UpdateDataFileInfo:		
			break;
		default:
			throw new Error("This case is impossible!");
			
	}
}


}  
